@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -164,7 +164,7 @@ sub Run {
         );
     }
 
-    $LayoutObject->ErrorScreen(
+    return $LayoutObject->ErrorScreen(
         Message => Translatable('Invalid Subaction.'),
     );
 }
