@@ -227,9 +227,13 @@ sub StatsParamsWidget {
 
         # load static module
         my $Params = $Kernel::OM->Get('Kernel::System::Stats')->GetParams( StatID => $StatID );
+
+        return if !$Params;
+
         $LayoutObject->Block(
             Name => 'Static',
         );
+
         PARAMITEM:
         for my $ParamItem ( @{$Params} ) {
             $LayoutObject->Block(
@@ -287,12 +291,15 @@ sub StatsParamsWidget {
                     my @Values = keys( %{ $ObjectAttribute->{Values} } );
                     $ObjectAttribute->{SelectedValues} = \@Values;
                 }
-                for ( @{ $ObjectAttribute->{SelectedValues} } ) {
+
+                VALUE:
+                for my $Value ( @{ $ObjectAttribute->{SelectedValues} } ) {
                     if ( $ObjectAttribute->{Values} ) {
-                        $ValueHash{$_} = $ObjectAttribute->{Values}->{$_};
+                        next VALUE if !defined $ObjectAttribute->{Values}->{$Value};
+                        $ValueHash{$Value} = $ObjectAttribute->{Values}->{$Value};
                     }
                     else {
-                        $ValueHash{Value} = $_;
+                        $ValueHash{Value} = $Value;
                     }
                 }
 
@@ -350,11 +357,15 @@ sub StatsParamsWidget {
 
                         my @FixedAttributes;
 
-                        for (@Sorted) {
-                            my $Value = $ValueHash{$_};
+                        ELEMENT:
+                        for my $Element (@Sorted) {
+                            my $Value = $ValueHash{$Element};
                             if ( $ObjectAttribute->{Translation} ) {
-                                $Value = $LayoutObject->{LanguageObject}->Translate( $ValueHash{$_} );
+                                $Value = $LayoutObject->{LanguageObject}->Translate( $ValueHash{$Element} );
                             }
+
+                            next ELEMENT if !defined $Value;
+
                             push @FixedAttributes, $Value;
                         }
 
@@ -406,7 +417,7 @@ sub StatsParamsWidget {
                             TreeView       => $ObjectAttribute->{TreeView} || 0,
                             Sort           => scalar $ObjectAttribute->{Sort},
                             SortIndividual => scalar $ObjectAttribute->{SortIndividual},
-                            SelectedID     => [ $LocalGetArray->( Param => $ElementName ) ],
+                            SelectedID     => $LocalGetParam->( Param => $ElementName ),
                             Class          => 'Modernize',
                         );
                         $LayoutObject->Block(
