@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -11,10 +11,13 @@ package Kernel::System::Crypt::PGP;
 use strict;
 use warnings;
 
+use Kernel::System::VariableCheck qw(:all);
+
 our @ObjectDependencies = (
     'Kernel::Config',
     'Kernel::System::Encode',
     'Kernel::System::FileTemp',
+    'Kernel::System::CheckItem',
     'Kernel::System::Log',
     'Kernel::System::Main',
     'Kernel::System::Time',
@@ -34,6 +37,27 @@ This is a sub module of Kernel::System::Crypt and contains all pgp functions.
 
 =cut
 
+sub new {
+    my ( $Type, %Param ) = @_;
+
+    # allocate new hash for object
+    my $Self = {};
+    bless( $Self, $Type );
+
+    $Self->{Debug} = $Param{Debug} || 0;
+
+    # check if module is enabled
+    return 0 if !$Kernel::OM->Get('Kernel::Config')->Get('PGP');
+
+    # call init()
+    $Self->_Init();
+
+    # check working ENV
+    return 0 if $Self->Check();
+
+    return $Self;
+}
+
 =item Check()
 
 check if environment is working
@@ -78,7 +102,6 @@ crypt a message
 sub Crypt {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     for my $ParamName (qw( Message Key )) {
         if ( !$Param{$ParamName} ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
@@ -89,9 +112,6 @@ sub Crypt {
         }
     }
 
-    # Quote the key parameter before passing it to the shell.
-    my $QuotedKey = $Self->_QuoteShellArgument( $Param{Key} );
-
     $Kernel::OM->Get('Kernel::System::Encode')->EncodeOutput( \$Param{Message} );
 
     # get temp file object
@@ -103,7 +123,7 @@ sub Crypt {
 
     my ( $FHCrypt, $FilenameCrypt ) = $FileTempObject->TempFile();
     close $FHCrypt;
-    my $GPGOptions = "--always-trust --yes --encrypt --armor -o $FilenameCrypt -r $QuotedKey $Filename";
+    my $GPGOptions = "--always-trust --yes --encrypt --armor -o $FilenameCrypt -r $Param{Key} $Filename";
     my $LogMessage = qx{$Self->{GPGBin} $GPGOptions 2>&1};
 
     # get crypted content
@@ -132,7 +152,6 @@ The returned hash %Result has the following keys:
 sub Decrypt {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     for (qw(Message)) {
         if ( !defined( $Param{$_} ) ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
@@ -186,7 +205,6 @@ sign a message
 sub Sign {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     for (qw(Message Key)) {
         if ( !$Param{$_} ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
@@ -224,11 +242,19 @@ sub Sign {
     print $FHPhrase $Pw;
     close $FHPhrase;
 
-    # Quote the key parameter before passing it to the shell.
-    my $QuotedKey = $Self->_QuoteShellArgument( $Param{Key} );
+    my $Quiet = '';
 
-    my $GPGOptions = qq{--passphrase-fd 0 --default-key $QuotedKey -o $FileSign $SigType $Filename};
-    my $LogMessage = qx{$Self->{GPGBin} $GPGOptions <$FilePhrase 2>&1};
+    # GnuPG 2.1 (and higher) may send info messages about used default keys to STDERR, which leads to problems.
+    if (
+        IsHashRefWithData( $Self->{Version} )
+        && sprintf( "%.3d%.3d", $Self->{Version}->{Major}, $Self->{Version}->{Minor} ) >= 2_001
+        )
+    {
+        $Quiet = '--quiet --batch --pinentry-mode=loopback';
+    }
+
+    my $GPGOptions = qq{$Quiet --passphrase-fd 0 -o $FileSign --default-key $Param{Key} $SigType $Filename};
+    my $LogMessage = qx{$Self->{GPGBin} $GPGOptions < $FilePhrase 2>&1};
 
     # error
     if ($LogMessage) {
@@ -279,7 +305,6 @@ The returned hash %Result has the following keys:
 sub Verify {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     if ( !$Param{Message} ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
@@ -593,7 +618,7 @@ sub Verify {
 
 =item KeySearch()
 
-returns a array with serach result (private and public keys)
+returns a array with search result (private and public keys)
 
     my @Keys = $CryptObject->KeySearch(
         Search => 'something to search'
@@ -624,7 +649,7 @@ returns an array with search result (private keys)
 sub PrivateKeySearch {
     my ( $Self, %Param ) = @_;
 
-    my $Search         = $Self->_QuoteShellArgument( $Param{Search} ) || '';
+    my $Search         = $Param{Search} || '';
     my $GPGOptions     = "--list-secret-keys --with-fingerprint --with-colons $Search";
     my @GPGOutputLines = qx{$Self->{GPGBin} $GPGOptions 2>&1};
 
@@ -644,7 +669,7 @@ returns an array with search result (public keys)
 sub PublicKeySearch {
     my ( $Self, %Param ) = @_;
 
-    my $Search         = $Self->_QuoteShellArgument( $Param{Search} ) || '';
+    my $Search         = $Param{Search} || '';
     my $GPGOptions     = "--list-keys --with-fingerprint --with-colons $Search";
     my @GPGOutputLines = qx{$Self->{GPGBin} $GPGOptions 2>&1};
 
@@ -664,8 +689,8 @@ returns public key in ascii
 sub PublicKeyGet {
     my ( $Self, %Param ) = @_;
 
-    my $QuotedKey = $Self->_QuoteShellArgument( $Param{Key} ) || '';
-    my $LogMessage = qx{$Self->{GPGBin} --export --armor $QuotedKey 2>&1};
+    my $Key = quotemeta( $Param{Key} || '' );
+    my $LogMessage = qx{$Self->{GPGBin} --export --armor $Key 2>&1};
     my $PublicKey;
     if ( $LogMessage =~ /nothing exported/i ) {
         $LogMessage =~ s/\n//g;
@@ -705,10 +730,34 @@ returns secret key in ascii
 sub SecretKeyGet {
     my ( $Self, %Param ) = @_;
 
-    my $QuotedKey = $Self->_QuoteShellArgument( $Param{Key} ) || '';
+    my $Key = quotemeta( $Param{Key} || '' );
 
-    my $LogMessage = qx{$Self->{GPGBin} --export-secret-keys --armor $QuotedKey 2>&1};
-    my $SecretKey  = '';
+    my $LogMessage = '';
+
+    # GnuPG 2.1 (and higher) asks via pinentry for the key passphrase. We suppress that behavior by passing the phrase
+    # via STDIN from temporary file (--passphrase-fd 0 / file descriptor 0).
+    if (
+        IsHashRefWithData( $Self->{Version} )
+        && sprintf( "%.3d%.3d", $Self->{Version}->{Major}, $Self->{Version}->{Minor} ) >= 2_001
+        )
+    {
+        my %PasswordHash = %{ $Kernel::OM->Get('Kernel::Config')->Get('PGP::Key::Password') };
+        my $Password = $PasswordHash{$Key} || '';
+
+        my ( $FH, $Filename ) = $Kernel::OM->Get('Kernel::System::FileTemp')->TempFile();
+        print $FH $Password;
+        close $FH;
+
+        $LogMessage
+            = qx{$Self->{GPGBin} --batch --pinentry-mode=loopback --export-secret-keys --passphrase-fd 0 --armor --decrypt <$Filename 2>&1};
+    }
+
+    # GnuPG 2.0 (and lower)
+    else {
+        $LogMessage = qx{$Self->{GPGBin} --export-secret-keys --armor $Key 2>&1};
+    }
+
+    my $SecretKey = '';
 
     if ( $LogMessage =~ /nothing exported/i ) {
         $LogMessage =~ s/\n//g;
@@ -746,7 +795,6 @@ remove public key from key ring
 sub PublicKeyDelete {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     if ( !$Param{Key} ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
@@ -755,9 +803,9 @@ sub PublicKeyDelete {
         return;
     }
 
-    my $QuotedKey  = $Self->_QuoteShellArgument( $Param{Key} ) || '';
+    my $Key        = quotemeta( $Param{Key} || '' );
     my $GPGOptions = '--status-fd 1';
-    my $Message    = qx{$Self->{GPGBin} $GPGOptions --delete-key $QuotedKey 2>&1};
+    my $Message    = qx{$Self->{GPGBin} $GPGOptions --delete-key $Key 2>&1};
 
     my %LogMessage = $Self->_HandleLog( LogString => $Message );
 
@@ -786,7 +834,6 @@ remove secret key from key ring
 sub SecretKeyDelete {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     if ( !$Param{Key} ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
@@ -842,7 +889,6 @@ add key to key ring
 sub KeyAdd {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     if ( !$Param{Key} ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
@@ -893,13 +939,28 @@ sub _Init {
         $Self->{GPGBin} = "LC_MESSAGES=POSIX $Self->{GPGBin} $Self->{Options}";
     }
 
+    # determine active GnuPG version
+    my $VersionString = '';
+
+    eval {
+        $VersionString = `$Self->{GPGBin} --version`;
+    };
+
+    if ( $VersionString =~ m{ gpg [ ]+ \(.+?\) [ ]+ (\d+)\.(\d+)\.(\d+) }smx ) {
+        $Self->{Version} = {
+            Major  => $1,
+            Minor  => $2,
+            Patch  => $3,
+            String => $1 . '.' . $2 . '.' . $3,
+        };
+    }
+
     return $Self;
 }
 
 sub _DecryptPart {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     for (qw(Key Password Filename)) {
         if ( !defined( $Param{$_} ) ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
@@ -910,20 +971,36 @@ sub _DecryptPart {
         }
     }
 
-    # get temp file object
     my $FileTempObject = $Kernel::OM->Get('Kernel::System::FileTemp');
 
+    # temp file for decrypted message
     my ( $FHDecrypt, $FileDecrypt ) = $FileTempObject->TempFile();
     close $FHDecrypt;
+
+    # temp file for passphrase
     my ( $FHPhrase, $FilePhrase ) = $FileTempObject->TempFile();
     print $FHPhrase $Param{Password};
     close $FHPhrase;
 
-    # Quote the filename parameter before passing it to the shell.
-    my $QuotedFilename = $Self->_QuoteShellArgument( $Param{Filename} );
+    my $LogMessage = '';
+
+    # GnuPG 2.1 (and higher)
+    if (
+        IsHashRefWithData( $Self->{Version} )
+        && sprintf( "%.3d%.3d", $Self->{Version}->{Major}, $Self->{Version}->{Minor} ) >= 2_001
+        )
+    {
+        my $GPGOptions
+            = qq{--batch --pinentry-mode=loopback --passphrase-fd 0 --armor -o $FileDecrypt --decrypt $Param{Filename}};
+        $LogMessage = qx{$Self->{GPGBin} $GPGOptions < $FilePhrase 2>&1};
+    }
+
+    # GnuPG 2.0 (and lower)
+    else {
+        my $GPGOptions = qq{--batch --passphrase-fd 0 --yes --decrypt -o $FileDecrypt $Param{Filename}};
+        $LogMessage = qx{$Self->{GPGBin} $GPGOptions <$FilePhrase 2>&1};
+    }
 
-    my $GPGOptions = qq{--batch --passphrase-fd 0 --yes --decrypt -o $FileDecrypt $QuotedFilename};
-    my $LogMessage = qx{$Self->{GPGBin} $GPGOptions <$FilePhrase 2>&1};
     if ( $LogMessage =~ /failed/i ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'notice',
@@ -958,7 +1035,6 @@ Clean and build the log
 sub _HandleLog {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     for (qw(LogString)) {
         if ( !defined( $Param{$_} ) ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
@@ -1060,6 +1136,20 @@ sub _ParseGPGKeyList {
             $Key{Expires}          = $Fields[6] || 'never';
             $Key{Identifier}       = $Fields[9];
             $Key{IdentifierMaster} = $Fields[9];
+
+            if ( $Key{Expires} eq 'never' || $Key{Status} ne 'good' ) {
+                next LINE;
+            }
+
+            # Status is good, but let's make sure the key isn't expired.
+            my $CurrentSystemTime = $TimeObject->SystemTime();
+            my $ExpiresKeyTime    = $TimeObject->TimeStamp2SystemTime(
+                String => $Key{Expires} . ' 23:59:59',
+            );
+
+            if ( $CurrentSystemTime >= $ExpiresKeyTime ) {
+                $Key{Status} = 'expired';
+            }
         }
 
         # skip anything before we've seen the first key
@@ -1086,7 +1176,11 @@ sub _ParseGPGKeyList {
             # only use last 8 chars of key-ID in order to be compatible with previous parser
             $Key{KeyPrivate} = substr( $Fields[4], -8, 8 );
         }
-        elsif ( $Type eq 'fpr' ) {
+
+        # The public and secret key information will be exposed at first. Since GnugPG 2 (and higher)
+        # the sub-key fingerprint will be displayed as well (--list-keys --with-colons), which leads
+        # to overwriting of the main information. Therefore we just collect the first fingerprint.
+        elsif ( $Type eq 'fpr' && !$Key{Fingerprint} && !$Key{Fingerprint} ) {
             $Key{FingerprintShort} = $Fields[9];
 
             # add fingerprint in standard format, too
@@ -1128,7 +1222,6 @@ sub _ParseGPGKeyList {
 sub _CryptedWithKey {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     if ( !$Param{File} ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
@@ -1137,9 +1230,6 @@ sub _CryptedWithKey {
         return;
     }
 
-    # Quote the file parameter before passing it to the shell.
-    my $QuotedFile = $Self->_QuoteShellArgument( $Param{File} );
-
     # This is a bit tricky: all we actually want is the list of keys that this message has been
     # encrypted for, but gpg does not seem to offer a way to just get these.
     # So we simply try to decrypt with an incorrect passphrase, which of course fails, but still
@@ -1148,7 +1238,7 @@ sub _CryptedWithKey {
     my ( $FHPhrase, $FilePhrase ) = $Kernel::OM->Get('Kernel::System::FileTemp')->TempFile();
     print $FHPhrase '_no_this_is_not_the_@correct@_passphrase_';
     close $FHPhrase;
-    my $GPGOptions     = qq{--batch --passphrase-fd 0 --always-trust --yes --decrypt $QuotedFile};
+    my $GPGOptions     = qq{--batch --passphrase-fd 0 --always-trust --yes --decrypt $Param{File}};
     my @GPGOutputLines = qx{$Self->{GPGBin} $GPGOptions <$FilePhrase 2>&1};
 
     my @Keys;
@@ -1165,39 +1255,6 @@ sub _CryptedWithKey {
     return @Keys;
 }
 
-=item _QuoteShellArgument()
-
-Quote passed string to be safe to use as a shell argument.
-
-    my $Result = $Self->_QuoteShellArgument(
-        "Safe string for 'shell arguments'."   # string to quote
-    );
-
-Returns quoted string if supplied or undef otherwise:
-
-    $Result = <<'EOS';
-'Safe string for '"'"'shell arguments'"'"'.'
-EOS
-
-=cut
-
-sub _QuoteShellArgument {
-    my ( $Self, $String ) = @_;
-
-    # Only continue with quoting if we received a valid string.
-    if ($String) {
-
-        # Encase any single quotes in double quotes, and glue them together with single quotes.
-        #   Please see https://stackoverflow.com/a/1250279 for more information.
-        $String =~ s/'/'"'"'/g;
-
-        # Enclose the string in single quotes.
-        return "'$String'";
-    }
-
-    return;
-}
-
 1;
 
 =end Internal:
